
#ifndef __GPX_H__
#define __GPX_H__

#include <glib.h>
#include <glib-object.h>
#include <float.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <gtk/gtk.h>

G_BEGIN_DECLS


#define GPX_TYPE_POINT (gpx_point_get_type ())
#define GPX_POINT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GPX_TYPE_POINT, GpxPoint))
#define GPX_POINT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GPX_TYPE_POINT, GpxPointClass))
#define GPX_IS_POINT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GPX_TYPE_POINT))
#define GPX_IS_POINT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GPX_TYPE_POINT))
#define GPX_POINT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GPX_TYPE_POINT, GpxPointClass))

typedef struct _GpxPoint GpxPoint;
typedef struct _GpxPointClass GpxPointClass;
typedef struct _GpxPointPrivate GpxPointPrivate;

#define GPX_TYPE_TRACK (gpx_track_get_type ())
#define GPX_TRACK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GPX_TYPE_TRACK, GpxTrack))
#define GPX_TRACK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GPX_TYPE_TRACK, GpxTrackClass))
#define GPX_IS_TRACK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GPX_TYPE_TRACK))
#define GPX_IS_TRACK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GPX_TYPE_TRACK))
#define GPX_TRACK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GPX_TYPE_TRACK, GpxTrackClass))

typedef struct _GpxTrack GpxTrack;
typedef struct _GpxTrackClass GpxTrackClass;
typedef struct _GpxTrackPrivate GpxTrackPrivate;

#define GPX_TYPE_FILE (gpx_file_get_type ())
#define GPX_FILE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GPX_TYPE_FILE, GpxFile))
#define GPX_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GPX_TYPE_FILE, GpxFileClass))
#define GPX_IS_FILE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GPX_TYPE_FILE))
#define GPX_IS_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GPX_TYPE_FILE))
#define GPX_FILE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GPX_TYPE_FILE, GpxFileClass))

typedef struct _GpxFile GpxFile;
typedef struct _GpxFileClass GpxFileClass;
typedef struct _GpxFilePrivate GpxFilePrivate;

#define GPX_TYPE_GRAPH (gpx_graph_get_type ())
#define GPX_GRAPH(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GPX_TYPE_GRAPH, GpxGraph))
#define GPX_GRAPH_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GPX_TYPE_GRAPH, GpxGraphClass))
#define GPX_IS_GRAPH(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GPX_TYPE_GRAPH))
#define GPX_IS_GRAPH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GPX_TYPE_GRAPH))
#define GPX_GRAPH_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GPX_TYPE_GRAPH, GpxGraphClass))

typedef struct _GpxGraph GpxGraph;
typedef struct _GpxGraphClass GpxGraphClass;
typedef struct _GpxGraphPrivate GpxGraphPrivate;

/**
 * Represents a point in the track or a waypoint. 
 */
struct _GpxPoint {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GpxPointPrivate * priv;
	double lat;
	double lon;
	double lat_dec;
	double lon_dec;
	double distance;
	double elevation;
	char* time;
	double speed;
};

struct _GpxPointClass {
	GTypeClass parent_class;
	void (*finalize) (GpxPoint *self);
};

/**
 * This class represents a Track in a gpx file.
 * The tracks contains the points connecting everything together. 
 * Info like total distance, average speed, moving speed/time etc are available.
 */
struct _GpxTrack {
	GObject parent_instance;
	GpxTrackPrivate * priv;
	double total_distance;
	double max_speed;
	GList* points;
	GpxPoint* top;
	GpxPoint* bottom;
};

struct _GpxTrackClass {
	GObjectClass parent_class;
};

/**
 * This is the top level class representing the gpx file it self.
 * This contains a list of tracks and waypoings.
 */
struct _GpxFile {
	GObject parent_instance;
	GpxFilePrivate * priv;
	char* filename;
	GList* tracks;
	GList* waypoints;
	GList* routes;
};

struct _GpxFileClass {
	GObjectClass parent_class;
};

struct _GpxGraph {
	GtkEventBox parent_instance;
	GpxGraphPrivate * priv;
	GpxTrack* track;
};

struct _GpxGraphClass {
	GtkEventBoxClass parent_class;
};


gpointer gpx_point_ref (gpointer instance);
void gpx_point_unref (gpointer instance);
GParamSpec* gpx_param_spec_point (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void gpx_value_set_point (GValue* value, gpointer v_object);
gpointer gpx_value_get_point (const GValue* value);
GType gpx_point_get_type (void);
void gpx_point_set_position (GpxPoint* self, double lat, double lon);
time_t gpx_point_get_time (GpxPoint* self);
GpxPoint* gpx_point_new (void);
GpxPoint* gpx_point_construct (GType object_type);
const char* gpx_point_get_name (GpxPoint* self);
void gpx_point_set_name (GpxPoint* self, const char* value);
GType gpx_track_get_type (void);
void gpx_track_add_point (GpxTrack* self, GpxPoint* point);
double gpx_track_get_track_average (GpxTrack* self);
double gpx_track_calculate_point_to_point_speed (GpxTrack* self, GpxPoint* a, GpxPoint* b);
time_t gpx_track_get_total_time (GpxTrack* self);
double gpx_track_calculate_moving_average (GpxTrack* self, GpxPoint* start, GpxPoint* stop, time_t* moving_time);
GpxTrack* gpx_track_new (void);
GpxTrack* gpx_track_construct (GType object_type);
const char* gpx_track_get_name (GpxTrack* self);
void gpx_track_set_name (GpxTrack* self, const char* value);
GType gpx_file_get_type (void);
GpxFile* gpx_file_new (const char* filename);
GpxFile* gpx_file_construct (GType object_type, const char* filename);
GType gpx_graph_get_type (void);
GpxGraph* gpx_graph_new (void);
GpxGraph* gpx_graph_construct (GType object_type);
void gpx_graph_set_track (GpxGraph* self, GpxTrack* track);
gint gpx_graph_get_smooth_factor (GpxGraph* self);
void gpx_graph_set_smooth_factor (GpxGraph* self, gint value);


G_END_DECLS

#endif
